'use strict';

/**
 * Use typedoc to read information about the component
 */

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var fs = require('fs');

var typedoc = require('typedoc');
var options = {
    'mode': 'modules',
    'theme': 'default',
    'ignoreCompilerErrors': 'true',
    'experimentalDecorators': 'true',
    'emitDecoratorMetadata': 'true',
    'target': 'ES6',
    'moduleResolution': 'node',
    'preserveConstEnums': 'true',
    'stripInternal': 'true',
    'suppressExcessPropertyErrors': 'true',
    'suppressImplicitAnyIndexErrors': 'true',
    'module': 'commonjs'
};

/**
 * Get the component raw data from typedoc
 */
function getComponentData(sourceData, callback) {
    var app = new typedoc.Application(options),
        files_array = ['./node_modules/carte-blanche-angular2/tmp/component.ts'];
    try {
        var content = fs.writeFileSync('./node_modules/carte-blanche-angular2/tmp/component.ts', sourceData);
    } catch (exception) {
        console.error(exception);
    }
    // lauch typedoc
    app.generateJson(files_array, './node_modules/carte-blanche-angular2/tmp/out.json');
    return fs.readFileSync('./node_modules/carte-blanche-angular2/tmp/out.json', 'utf8');
}

/**
 * Resolve and clean the component data that comes from typedoc
 */
function checkChildren(object) {
    var analyzedObject = {
        elements: []
    };
    // go through every child and fetch component decorators
    if (typeof object.children != 'undefined') {
        object.children.forEach(function (val) {
            var componentInfo = getComponentInfo(val);
            var componentDecorators = getChildComponentDecorators(componentInfo);
            var componentInputs = getChildComponentInputs(componentInfo);

            // Create the element and put it in the array
            var element = {
                componentInfo: componentInfo,
                componentDecorators: componentDecorators,
                inputs: componentInputs
            };
            analyzedObject.elements = [].concat(_toConsumableArray(analyzedObject.elements), [element]);
            //checkChildren(val);
        });
    }
    return analyzedObject;
}

/**
 * Get some basic info on the component like name, children properties etc.
 */
function getComponentInfo(object) {
    if (typeof object.children != 'undefined' && object.children.length > 0) {
        // Go throguh each children and return the class that has a component decorator (the component)
        // right now only works if the decorator is right on top of the component !
        var componentArray = object.children.filter(function (val) {
            return val.kindString == 'Class' && typeof val.decorators.length != 'undefined' && val.decorators[0].name == 'Component';
        });
        if (componentArray.length > 0) {
            // Return the first component. There shouldn't be multiple components in one file.
            return componentArray[0];
        }
    }
}

/**
 * Get the component decorators
 * This will be used to render the component in the frontend dynamically.
 */
function getChildComponentDecorators(component) {
    var componentDecorators = [];
    if (typeof component.decorators != 'undefied' && component.decorators.length > 0) {
        component.decorators.forEach(function (val) {
            if (val.name = 'Component') {
                componentDecorators = [].concat(_toConsumableArray(componentDecorators), [val]);
            }
        });
    }
    return componentDecorators;
}

/**
 * Get the component input properties
 * This will be used to render the correspondent component inputs and variations of them
 */
function getChildComponentInputs(component) {
    var componentInputs = [];

    if (typeof component.children != 'undefied' && component.children.length > 0) {
        var objectChildren = component.children.filter(function (item) {
            return item.kindString == 'Property' && _typeof(item.decorators != 'undefined' && item.decorators.length > 0);
        }).forEach(function (input) {
            componentInputs = [].concat(_toConsumableArray(componentInputs), [input]);
        });
    };
    return componentInputs;
}

/**
 * Get the clean component information including component info, inputs 
 * and component decorator.
 */
function resolveComponentInfo(data) {
    return checkChildren(data);
}

// export functions
module.exports.getComponentData = getComponentData;
module.exports.resolveComponentInfo = resolveComponentInfo;